"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFilePath = exports.parseFileName = exports.fileNameOf = exports.IgnoredTags = exports.convertor = exports.Empty_URL = exports.sourcePathOf = exports.parseMeta = exports.parseContent = exports.likeOf = exports.LinkSelector = exports.MediaSelector = exports.AttributeKey = void 0;
const tslib_1 = require("tslib");
require("array-unique-proposal");
const turndown_1 = tslib_1.__importDefault(require("turndown"));
const turndown_plugin_gfm_1 = require("turndown-plugin-gfm");
const node_toolkit_1 = require("@tech_query/node-toolkit");
const file_type_1 = require("file-type");
const path_1 = require("path");
const config_1 = require("./config");
exports.AttributeKey = {
    '#': 'id',
    '.': 'class'
}, exports.MediaSelector = 'img, audio, video', exports.LinkSelector = 'a[href], area[href]';
function likeOf(selector) {
    const key = exports.AttributeKey[selector[0]];
    return key ? `[${key}*="${selector.slice(1)}" i]` : selector;
}
exports.likeOf = likeOf;
function parseContent(document, rootSelector = '') {
    let root;
    for (const selector of [rootSelector, ...config_1.body_tag])
        if (selector && (root = document.querySelector(selector)))
            break;
    for (const element of document.querySelectorAll(exports.IgnoredTags + ''))
        element.remove();
    for (const element of document.querySelectorAll('[style*="display:"]')) {
        const { display, visibility, opacity, width, height } = document.defaultView.getComputedStyle(element);
        if (!element.matches(exports.MediaSelector) &&
            (display === 'none' ||
                visibility === 'hidden' ||
                !(opacity || width || height)))
            element.remove();
    }
    return root;
}
exports.parseContent = parseContent;
function parseMeta(document, selectorMap) {
    const data = {};
    for (const key in selectorMap)
        for (const selector of selectorMap[key]) {
            const tags = document.querySelectorAll(selector);
            if (!tags[0])
                continue;
            const value = Array.from(tags, tag => (tag.remove(), tag.textContent.trim()))
                .uniqueBy()
                .filter(Boolean);
            data[key] = key.endsWith('s') ? value : value[0];
            break;
        }
    return data;
}
exports.parseMeta = parseMeta;
function sourcePathOf({ dataset, src }) {
    for (const key in dataset)
        try {
            return new URL(dataset[key]);
        }
        catch (_a) {
            //
        }
    return new URL(src);
}
exports.sourcePathOf = sourcePathOf;
exports.Empty_URL = /^(#+|javascript:.+)$/;
exports.convertor = new turndown_1.default({
    headingStyle: 'atx',
    hr: '---',
    bulletListMarker: '-',
    codeBlockStyle: 'fenced',
    linkStyle: 'referenced'
});
exports.IgnoredTags = [
    'meta',
    'link',
    'script',
    'form',
    'fieldset',
    'legend',
    'label',
    'input',
    'button'
];
exports.convertor
    .use(turndown_plugin_gfm_1.gfm)
    .addRule('user_interface', {
    filter: exports.IgnoredTags,
    replacement: () => ''
})
    .addRule('non_url', {
    filter: node => ['a', 'area'].includes(node.nodeName.toLowerCase()) &&
        exports.Empty_URL.test(node.getAttribute('href')),
    replacement: () => ''
});
function fileNameOf(raw) {
    return raw.replace(/[/\\:*?'"<>|.#\s]+/g, '-');
}
exports.fileNameOf = fileNameOf;
function parseFileName(path) {
    const name_parts = path
        .split('/')
        .filter(Boolean)
        .slice(-1)[0]
        .split('.')
        .filter(Boolean);
    return {
        base: name_parts.slice(0, -1).join('.'),
        ext: name_parts[1] && name_parts.slice(-1)[0].match(/^\w+/)[0]
    };
}
exports.parseFileName = parseFileName;
async function createFilePath(data, raw_path, target_root = '') {
    var _a;
    const { base, ext } = parseFileName(raw_path);
    const type = ext || ((_a = (await (0, file_type_1.fromBuffer)(data))) === null || _a === void 0 ? void 0 : _a.ext);
    let name = base || (0, node_toolkit_1.uniqueID)();
    if (type)
        name += `.${type}`;
    return (0, path_1.join)(target_root, decodeURI(name)).replace(/\\+/g, '/');
}
exports.createFilePath = createFilePath;
